'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _require = require('nocms-events'),
    listenToGlobal = _require.listenToGlobal,
    stopListenToGlobal = _require.stopListenToGlobal,
    triggerGlobal = _require.triggerGlobal,
    clearEventGlobal = _require.clearEventGlobal;

var stores = {};

var subscribe = function subscribe(name, func) {
  listenToGlobal('store:' + name, func);
};

var unsubscribe = function unsubscribe(name, func) {
  stopListenToGlobal('store:' + name, func);
};

var createStore = function createStore(name, value, func) {
  var initialValue = value || {};
  var cb = func;
  if (typeof initialValue === 'function') {
    cb = initialValue;
    initialValue = {};
  }

  if (!stores[name]) {
    stores[name] = Object.assign({}, initialValue) || {};
  } else {
    stores[name] = Object.assign(initialValue, stores[name]);
  }

  if (typeof cb === 'function') {
    subscribe(name, cb);
  }
};

var deleteStore = function deleteStore(name) {
  delete stores[name];

  clearEventGlobal('store:' + name);
};

var remove = function remove(name) {
  if (name.substr(-1) === '*') {
    var prefix = name.substr(0, name.length - 1);
    Object.keys(stores).filter(function (store) {
      return store.indexOf(prefix) === 0;
    }).forEach(function (store) {
      deleteStore(store);
    });
  } else {
    deleteStore(name);
  }
};

var getStore = function getStore(name) {
  return stores[name];
};

var patch = function patch(name, obj) {
  if (!stores[name] || obj === null) {
    return;
  }
  Object.keys(obj).forEach(function (prop) {
    if (_typeof(obj[prop]) !== 'object') {
      return;
    }
    if (typeof stores[name][prop] === 'undefined') {
      stores[name][prop] = {};
    }
    Object.keys(obj[prop]).forEach(function (field) {
      stores[name][prop][field] = obj[prop][field];
    });
  });
  triggerGlobal('store:' + name, stores[name], obj);
};

var update = function update(name, obj) {
  if (!stores[name] || obj === null) {
    return;
  }
  Object.keys(obj).forEach(function (prop) {
    stores[name][prop] = obj[prop];
  });
  triggerGlobal('store:' + name, stores[name], obj);
};

var clearStore = function clearStore(store) {
  delete stores[store];
  triggerGlobal('store:' + store, {}, {});
};

var clearAll = function clearAll() {
  Object.keys(stores).forEach(clearStore);
};

module.exports = {
  createStore: createStore,
  remove: remove,
  getStore: getStore,
  subscribe: subscribe,
  unsubscribe: unsubscribe,
  update: update,
  clearAll: clearAll,
  clearStore: clearStore,
  patch: patch
};
//# sourceMappingURL=index.js.map